use env_logger::{Builder, Target};
use log::LevelFilter;
use std::fs::OpenOptions;
use std::io::{self, Write};
use std::path::PathBuf;

/// Custom writer that writes either to stderr or to a file
pub(crate) struct LogWriter {
    file: Option<std::fs::File>,
}

impl LogWriter {
    fn new(file: Option<std::fs::File>) -> Self {
        Self { file }
    }
}

impl Write for LogWriter {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        match &mut self.file {
            Some(file) => file.write_all(buf)?,  // Write to file if specified
            None => io::stderr().write_all(buf)?, // Otherwise write to stderr
        }
        Ok(buf.len())
    }

    fn flush(&mut self) -> io::Result<()> {
        match &mut self.file {
            Some(file) => file.flush()?,
            None => io::stderr().flush()?,
        }
        Ok(())
    }
}

pub struct LogConfig {
    pub log_file: Option<PathBuf>,
    pub log_level: String,
    pub append_log: bool,
}

/// Initialize logging to either stderr or file (if specified)
pub fn init_logging(config: &LogConfig) -> Result<(), Box<dyn std::error::Error>> {
    let mut builder = Builder::from_default_env();

    // Set log level from command line argument
    let level = match config.log_level.to_lowercase().as_str() {
        "error" => LevelFilter::Error,
        "warn" => LevelFilter::Warn,
        "info" => LevelFilter::Info,
        "debug" => LevelFilter::Debug,
        "trace" => LevelFilter::Trace,
        _ => LevelFilter::Info,
    };
    builder.filter_level(level);

    // Format with timestamps, module path, and line numbers for debug/trace
    builder.format(move |buf, record| {
        let timestamp = chrono::Local::now().format("%Y-%m-%d %H:%M:%S%.3f");
        if level >= LevelFilter::Debug {
            writeln!(
                buf,
                "{} [{}] [{}:{}] - {}",
                timestamp,
                record.level(),
                record.module_path().unwrap_or("unknown"),
                record.line().unwrap_or(0),
                record.args()
            )
        } else {
            writeln!(
                buf,
                "{} [{}] - {}",
                timestamp,
                record.level(),
                record.args()
            )
        }
    });

    // Set up the writer for either file or console output
    let log_file = if let Some(log_path) = &config.log_file {
        let file = OpenOptions::new()
            .create(true)
            .write(true)
            .append(config.append_log)
            .truncate(!config.append_log)
            .open(log_path)?;

        // Write header to log file if not appending
        if !config.append_log {
            writeln!(
                &file,
                "=== Log started at {} ===",
                chrono::Local::now().format("%Y-%m-%d %H:%M:%S")
            )?;
        }
        Some(file)
    } else {
        None
    };

    // Create and set the writer
    let writer = LogWriter::new(log_file);
    builder.target(Target::Pipe(Box::new(writer)));

    builder.init();

    // Log initial message with configuration info
    log::info!(
        "Logging initialized (level: {}, output: {})",
        config.log_level,
        config.log_file
            .as_ref()
            .map(|p| format!("file: {}", p.display()))
            .unwrap_or_else(|| "console".to_string())
    );

    Ok(())
} 